@@ -111,7 +111,7 @@ class _ConverterCallable:
             os.makedirs(cls.debug_dir, exist_ok=True)
             cls.debug_dir_initialized = True
         for item in self._name_to_converter.values():
-            with open(item["abs_path"], "w") as f:
+            with open(item["abs_path"], "w", encoding="utf-8") as f:
                 f.write(item["code_str"])
 
         self._code_dumped = True
@@ -274,8 +274,14 @@ class BaseConversion(typing.Generic[CT]):
         ARG_USAGE = 8
         LABEL_USAGE = 16
         BREAKPOINT = 32
+        FUNCTION_OF_INPUT = 64
 
-    self_content_type = 0
+    self_content_type = ContentTypes.FUNCTION_OF_INPUT
+
+    class OutputHints:
+        NOT_NONE = 1
+
+    output_hints = 0
 
     def __init__(self):
         self._depends_on = {}
@@ -284,6 +290,10 @@ class BaseConversion(typing.Generic[CT]):
     def __hash__(self):
         return id(self)
 
+    def add_hint(self, hint: int):
+        self.output_hints |= hint
+        return self
+
     def check_dependency(self, b, for_piping=False):
         contents = self.contents if for_piping else self.self_content_type
         if contents & b.contents & self.ContentTypes.REDUCER:
@@ -497,6 +507,7 @@ class BaseConversion(typing.Generic[CT]):
         # signature should contain "data_" argument
         initial_code_input = "data_"
         debug = debug or (self.contents & self.ContentTypes.BREAKPOINT)
+        has_labels = self.contents & self.ContentTypes.NEW_LABEL
         ctx = self._init_ctx(debug=debug)
 
         converter_callable_cls = CodeGenerationOptionsCtx.get_option_value(
@@ -544,11 +555,13 @@ class BaseConversion(typing.Generic[CT]):
         code.add_line("global __naive_values__, __none__", 0)
         code.add_line("_naive = __naive_values__", 0)
         code.add_line("_none = __none__", 0)
-        code.add_line("_labels = {}", 0)
+        if has_labels:
+            code.add_line("_labels = {}", 0)
+        else:
+            code.add_line("_labels = _none", 0)
+
         code.add_line(
-            "return {}".format(
-                self.gen_code_and_update_ctx(initial_code_input, ctx)
-            ),
+            f"return {self.gen_code_and_update_ctx(initial_code_input, ctx)}",
             0,
         )
         main_converter = self._code_to_converter(
@@ -613,6 +626,9 @@ class BaseConversion(typing.Generic[CT]):
     def is_itself_callable(self) -> typing.Optional[bool]:
         pass
 
+    def is_independent(self) -> typing.Optional[bool]:
+        return not self.contents & self.ContentTypes.FUNCTION_OF_INPUT
+
     def call_like(self, *args, **kwargs):
         if self.is_itself_callable_like():
             return self.call(*args, **kwargs)
@@ -714,47 +730,78 @@ class BaseConversion(typing.Generic[CT]):
         return self.lte(b)
 
     def neg(self) -> "InlineExpr":
-        return InlineExpr("-{0}").pass_args(self)
+        return (
+            InlineExpr("-{0}")
+            .pass_args(self)
+            .add_hint(self.OutputHints.NOT_NONE)
+        )
 
     def __neg__(self) -> "InlineExpr":
         return self.neg()
 
     def add(self, arg) -> "InlineExpr":
-        return InlineExpr("{0} + {1}").pass_args(self, arg)
+        return (
+            InlineExpr("{0} + {1}")
+            .pass_args(self, arg)
+            .add_hint(self.OutputHints.NOT_NONE)
+        )
 
     def __add__(self, b) -> "InlineExpr":
         return self.add(b)
 
     def mul(self, arg) -> "InlineExpr":
-        return InlineExpr("{0} * {1}").pass_args(self, arg)
+        return (
+            InlineExpr("{0} * {1}")
+            .pass_args(self, arg)
+            .add_hint(self.OutputHints.NOT_NONE)
+        )
 
     def __mul__(self, b) -> "InlineExpr":
         return self.mul(b)
 
     def sub(self, arg) -> "InlineExpr":
-        return InlineExpr("{0} - {1}").pass_args(self, arg)
+        return (
+            InlineExpr("{0} - {1}")
+            .pass_args(self, arg)
+            .add_hint(self.OutputHints.NOT_NONE)
+        )
 
     def __sub__(self, b) -> "InlineExpr":
         return self.sub(b)
 
     def div(self, arg) -> "InlineExpr":
-        return InlineExpr("{0} / {1}").pass_args(self, arg)
+        return (
+            InlineExpr("{0} / {1}")
+            .pass_args(self, arg)
+            .add_hint(self.OutputHints.NOT_NONE)
+        )
 
     def __truediv__(self, b) -> "InlineExpr":
         return self.div(b)
 
     def mod(self, arg) -> "InlineExpr":
-        return InlineExpr("{0} % {1}").pass_args(self, arg)
+        return (
+            InlineExpr("{0} % {1}")
+            .pass_args(self, arg)
+            .add_hint(self.OutputHints.NOT_NONE)
+        )
 
     def __mod__(self, b) -> "InlineExpr":
         return self.mod(b)
 
     def floor_div(self, arg) -> "InlineExpr":
-        return InlineExpr("{0} // {1}").pass_args(self, arg)
+        return (
+            InlineExpr("{0} // {1}")
+            .pass_args(self, arg)
+            .add_hint(self.OutputHints.NOT_NONE)
+        )
 
     def __floordiv__(self, b) -> "InlineExpr":
         return self.floor_div(b)
 
+    def len(self) -> "BaseConversion":
+        return CallFunc(len, self)
+
     def filter(self, condition_conv, cast=_none) -> "BaseConversion":
         """Shortcut for calling :py:obj:`convtools.base.FilterConversion` on
         self"""
@@ -824,7 +871,7 @@ class BaseConversion(typing.Generic[CT]):
         label_output=None,
         **kwargs,
     ) -> "BaseConversion":
-        """Shortcut for PipeConversion"""
+        """Shortcut for :py:obj:`PipeConversion`"""
         return PipeConversion(
             self,
             next_conversion,
@@ -851,11 +898,10 @@ class BaseMethodConversion(BaseConversion):
 
     def __init__(self, self_conv):
         super().__init__()
-        self.self_conv = (
-            None
-            if self_conv is self._none
-            else self.ensure_conversion(self_conv)
-        )
+        if self_conv is self._none:
+            self.self_conv = None
+        else:
+            self.self_conv = self.ensure_conversion(self_conv)
 
     def get_self_and_input_code(
         self, code_input: str, ctx: dict
@@ -888,6 +934,10 @@ class NaiveConversion(BaseConversion):
     """
 
     _builtin_dict = globals()["__builtins__"]
+    self_content_type = (
+        BaseConversion.self_content_type
+        ^ BaseConversion.ContentTypes.FUNCTION_OF_INPUT
+    )
 
     def __init__(self, value: typing.Any, name_prefix="v"):
         """
@@ -918,15 +968,18 @@ class NaiveConversion(BaseConversion):
                 if f_name:
                     self.name_prefix = f_name
 
+    types_to_repr = {type(None), bool, int}
+
     def _gen_code_and_update_ctx(self, code_input, ctx):
         if self.code_str:
             return self.code_str
+
         value = self.value
+        value_type = type(value)
         if (
-            value is None
-            or isinstance(value, (bool, int))
-            or isinstance(value, str)
-            and len(value) < 255
+            value_type in self.types_to_repr
+            or value_type is str
+            and len(value) < 128
             and "%" not in value
             and "{" not in value
         ):
@@ -935,7 +988,7 @@ class NaiveConversion(BaseConversion):
             self.name_prefix, ctx, value
         )
         ctx["__naive_values__"][value_name] = value
-        return f"_naive['{value_name}']"
+        return f'_naive["{value_name}"]'
 
     def is_itself_callable_like(self) -> typing.Optional[bool]:
         return callable(self.value)
@@ -948,6 +1001,11 @@ class EscapedString(BaseConversion):
     """Defines the conversion which returns the result of running the
     python code, passed to init"""
 
+    self_content_type = (
+        BaseConversion.self_content_type
+        ^ BaseConversion.ContentTypes.FUNCTION_OF_INPUT
+    )
+
     def __init__(self, s):
         super().__init__()
         self.s = s
@@ -962,6 +1020,10 @@ class This(BaseConversion):
     Also, provided that you use this inside comprehension conversions,
     it references an item from an iterator."""
 
+    def __call__(self) -> "This":
+        """To allow using it as singleton"""
+        return self
+
     def _gen_code_and_update_ctx(self, code_input, ctx):
         return code_input
 
@@ -973,7 +1035,10 @@ class InputArg(BaseConversion):
     input arguments used in the conversion definition will be expected as
     keyword-only arguments (affecting the resulting converter signature)."""
 
-    self_content_type = BaseConversion.ContentTypes.ARG_USAGE
+    self_content_type = (
+        BaseConversion.self_content_type
+        | BaseConversion.ContentTypes.ARG_USAGE
+    ) ^ BaseConversion.ContentTypes.FUNCTION_OF_INPUT
 
     def __init__(self, arg_name: str):
         """
@@ -991,7 +1056,10 @@ class LabelConversion(BaseConversion):
     """Allows to reference a conversion result by label, after it was cached by
     :py:obj:`PipeConversion` or :py:obj:`BaseConversion.add_label`."""
 
-    self_content_type = BaseConversion.ContentTypes.LABEL_USAGE
+    self_content_type = (
+        BaseConversion.self_content_type
+        | BaseConversion.ContentTypes.LABEL_USAGE
+    ) ^ BaseConversion.ContentTypes.FUNCTION_OF_INPUT
 
     def __init__(self, label_name: str):
         """
@@ -1073,14 +1141,10 @@ class Or(BaseConversion):
         self.args = [self.ensure_conversion(a) for a in args]
 
     def _gen_code_and_update_ctx(self, code_input, ctx):
-        return "({})".format(
-            self.op.join(
-                [
-                    arg.gen_code_and_update_ctx(code_input, ctx)
-                    for arg in self.args
-                ]
-            )
+        code = self.op.join(
+            [arg.gen_code_and_update_ctx(code_input, ctx) for arg in self.args]
         )
+        return f"({code})"
 
 
 class And(Or):
@@ -1172,9 +1236,8 @@ class Not(BaseConversion):
         self.arg = self.ensure_conversion(arg)
 
     def _gen_code_and_update_ctx(self, code_input, ctx):
-        return "(not {})".format(
-            self.arg.gen_code_and_update_ctx(code_input, ctx)
-        )
+        code = self.arg.gen_code_and_update_ctx(code_input, ctx)
+        return f"(not {code})"
 
 
 class GetItem(BaseMethodConversion):
@@ -1283,6 +1346,11 @@ class Call(BaseMethodConversion):
     It takes both positional and keyword arguments to be passed.
     """
 
+    self_content_type = (
+        BaseConversion.self_content_type
+        ^ BaseConversion.ContentTypes.FUNCTION_OF_INPUT
+    )
+
     def __init__(self, *args, self_conv=BaseConversion._none, **kwargs):
         super().__init__(self_conv)
         self.args = [self.ensure_conversion(arg) for arg in args]
@@ -1301,14 +1369,14 @@ class Call(BaseMethodConversion):
                 for param in self.args
             ),
             (
-                "{}={}".format(k, v.gen_code_and_update_ctx(code_input, ctx))
+                f"{k}={v.gen_code_and_update_ctx(code_input, ctx)}"
                 for k, v in self.kwargs.items()
             ),
         )
         return f"{code_self}({','.join(params)})"
 
 
-def CallFunc(func, *args, **kwargs) -> "Call":
+def CallFunc(func, *args, **kwargs) -> "Call":  # pylint:disable=invalid-name
     """Shortcut to ``NaiveConversion(func).call(*args, **kwargs)``"""
     assert callable(func)
     return NaiveConversion(func).call(*args, **kwargs)
@@ -1318,7 +1386,7 @@ def _applier(func, args, kwargs):
     return func(*args, **kwargs)
 
 
-def ApplyFunc(func, args, kwargs) -> "Call":
+def ApplyFunc(func, args, kwargs) -> "Call":  # pylint:disable=invalid-name
     """Shortcut to ``NaiveConversion(func).apply(args, kwargs)``"""
     return NaiveConversion(_applier).call(func, args, kwargs)
 
@@ -1351,9 +1419,7 @@ class FilterConversion(BaseConversion):
             gen = GeneratorComp(GetItem(), where=condition_conv)
             result = NaiveConversion(cast).call(gen)
         else:
-            raise AssertionError(
-                "cannot cast generator to cast={}".format(cast)
-            )
+            raise AssertionError(f"cannot cast generator to cast={cast}")
         self.conversion = self.ensure_conversion(result)
 
     def as_type(self, callable_):
@@ -1397,6 +1463,11 @@ class InlineExpr(BaseConversion):
     """This conversion allows to avoid function call overhead.  It inlines a
     raw python code expression into the code of resulting conversion."""
 
+    self_content_type = (
+        BaseConversion.self_content_type
+        ^ BaseConversion.ContentTypes.FUNCTION_OF_INPUT
+    )
+
     def __init__(self, code_str):
         """
         Args:
@@ -1498,14 +1569,14 @@ class GeneratorComp(BaseComprehensionConversion):
         return super().as_type(callable_)
 
     def _gen_code_and_update_ctx(self, code_input, ctx):
-        return "({})".format(self.gen_generator_code(code_input, ctx))
+        return f"({self.gen_generator_code(code_input, ctx)})"
 
 
 class SetComp(BaseComprehensionConversion):
     """Generates python set comprehension code (obviously non-sortable)"""
 
     def _gen_code_and_update_ctx(self, code_input, ctx):
-        return "{%s}" % self.gen_generator_code(code_input, ctx)
+        return f"{{{self.gen_generator_code(code_input, ctx)}}}"
 
     def filter(self, condition_conv, cast=BaseConversion._none):
         if cast is self._none:
@@ -1518,7 +1589,7 @@ class ListComp(BaseComprehensionConversion):
     """Generates python list comprehension code."""
 
     def _gen_code_and_update_ctx(self, code_input, ctx):
-        return "[%s]" % self.gen_generator_code(code_input, ctx)
+        return f"[{self.gen_generator_code(code_input, ctx)}]"
 
     def filter(self, condition_conv, cast=BaseConversion._none):
         if cast is self._none:
@@ -1535,7 +1606,7 @@ class TupleComp(BaseComprehensionConversion):
     """Generates python tuple comprehension code."""
 
     def _gen_code_and_update_ctx(self, code_input, ctx):
-        return "tuple(%s)" % self.gen_generator_code(code_input, ctx)
+        return f"tuple({self.gen_generator_code(code_input, ctx)})"
 
     def filter(self, condition_conv, cast=BaseConversion._none):
         if cast is self._none:
@@ -1573,7 +1644,7 @@ class DictComp(BaseComprehensionConversion):
         return f"{key_code}: {value_code}"
 
     def _gen_code_and_update_ctx(self, code_input, ctx):
-        return "{%s}" % self.gen_generator_code(code_input, ctx)
+        return f"{{{self.gen_generator_code(code_input, ctx)}}}"
 
     def filter(self, condition_conv, cast=BaseConversion._none):
         if cast is self._none:
@@ -1596,6 +1667,11 @@ class DictComp(BaseComprehensionConversion):
 class BaseCollectionConversion(BaseConversion):
     """This is a base conversion of every collection"""
 
+    self_content_type = (
+        BaseConversion.self_content_type
+        ^ BaseConversion.ContentTypes.FUNCTION_OF_INPUT
+    )
+
     def __init__(self, *items):
         """
         Args:
@@ -1765,7 +1841,7 @@ class Set(BaseCollectionConversion):
     def gen_collection_from_items_code(
         self, joined_items_code, code_input, ctx
     ):
-        return "{%s}" % joined_items_code
+        return f"{{{joined_items_code}}}"
 
     def gen_collection_from_generator(self, generator_code, code_input, ctx):
         return f"set({generator_code})"
@@ -1825,10 +1901,8 @@ class Dict(BaseCollectionConversion):
 
     def gen_joined_items_code(self, code_input, ctx):
         return ",".join(
-            "{}:{}".format(
-                key.gen_code_and_update_ctx(code_input, ctx),
-                value.gen_code_and_update_ctx(code_input, ctx),
-            )
+            f"{key.gen_code_and_update_ctx(code_input, ctx)}:"
+            f"{value.gen_code_and_update_ctx(code_input, ctx)}"
             for key, value in self.key_value_pairs
         )
 
@@ -1838,7 +1912,7 @@ class Dict(BaseCollectionConversion):
     def gen_collection_from_items_code(
         self, joined_items_code, code_input, ctx
     ):
-        return "{%s}" % joined_items_code
+        return f"{{{joined_items_code}}}"
 
 
 class TakeWhile(BaseConversion):
@@ -2131,12 +2205,8 @@ class PipeConversion(BaseConversion):
         if self.label_input or self.label_output:
             label_input_code = (
                 "\n".join(
-                    "    _labels['{var_label}'] = {code_label}".format(
-                        var_label=label_name,
-                        code_label=label_conv.gen_code_and_update_ctx(
-                            var_input, ctx
-                        ),
-                    )
+                    f"    _labels['{label_name}'] = "
+                    f"{label_conv.gen_code_and_update_ctx(var_input, ctx)}"
                     for label_name, label_conv in self.label_input.items()
                 )
                 if self.label_input
@@ -2144,12 +2214,8 @@ class PipeConversion(BaseConversion):
             )
             label_output_code = (
                 "\n".join(
-                    "    _labels['{var_label}'] = {code_label}".format(
-                        var_label=label_name,
-                        code_label=label_conv.gen_code_and_update_ctx(
-                            var_result, ctx
-                        ),
-                    )
+                    f"    _labels['{label_name}'] = "
+                    f"{label_conv.gen_code_and_update_ctx( var_result, ctx)}"
                     for label_name, label_conv in self.label_output.items()
                 )
                 if self.label_output
@@ -2265,14 +2331,14 @@ if "pydevd" in sys.modules:  # pragma: no cover
 elif sys.version_info[:2] < (3, 7):  # pragma: no cover
 
     def debug_func(obj):
-        pdb.set_trace()
+        pdb.set_trace()  # pylint: disable=forgotten-debug-statement
         return obj
 
 
 else:  # pragma: no cover
 
     def debug_func(obj):
-        breakpoint()  # pylint: disable=undefined-variable # noqa: F821
+        breakpoint()  # pylint: disable=undefined-variable,forgotten-debug-statement # noqa: F821,E501
         return obj
 
 
@@ -2280,7 +2346,10 @@ class Breakpoint(BaseConversion):
     """Defines the conversion which wraps another one and puts a breakpoint
     after it"""
 
-    self_content_type = BaseConversion.ContentTypes.BREAKPOINT
+    self_content_type = (
+        BaseConversion.self_content_type
+        | BaseConversion.ContentTypes.BREAKPOINT
+    ) ^ BaseConversion.ContentTypes.FUNCTION_OF_INPUT
     debug_func = staticmethod(debug_func)
 
     def __init__(self, to_debug):
